@@ -1,7 +1,252 @@
 C
 C __________________________________________________________________
 C
-      subroutine mfunpl0(par,siq,grad,m,lpar,ngrad,pen,z,w,erg)
+      subroutine mfunpl(par,w,siq,g,m,lpar,n,z,erg)
+C
+C   model without isotropic compartment 
+C   with explicit parametrization of weights
+C
+C   code is restricted to m<=6
+C
+      implicit logical (a-z)
+      integer m,lpar,n
+      real*8 par(lpar),w(m),siq(n),g(3,n),z(n,m),erg
+      integer i,j
+      real*8 th,sw,sth,z1,p0,p1,d1,d2,d3
+      th = par(1)
+      DO i = 1,m
+         p0=par(2*i)
+         p1=par(2*i+1)
+         sth = sin(p0)
+         d1 = sth*cos(p1)
+         d2 = sth*sin(p1)
+         d3 = cos(p0)
+         DO j = 1,n
+            z1 = d1*g(1,j)+d2*g(2,j)+d3*g(3,j)
+            z(j,i) = exp(-th*z1*z1)
+         END DO
+      END DO
+      sw=0.d0
+      DO j=1,n
+         z1 = siq(j)
+         DO i=1,m
+            z1=z1-w(i)*z(j,i)
+         END DO
+         sw=sw+z1*z1
+      END DO
+      erg=sw
+      call rchkusr()
+      RETURN
+      END 
+C
+C __________________________________________________________________
+C
+      subroutine gmfunpl(par,w,siq,g,m,lpar,n,z,res,resd,dkgj,dkgj2,
+     1                 ddkdphig,ddkdetag,dzdpars,work1,work2,dfdparw)
+C
+C   analytical gradients for c(par,w)
+C   model without isotropic compartment 
+C   with explicit parametrization of weights
+C
+C   code is restricted to m<=6
+C
+      implicit logical (a-z)
+      integer m,lpar,n
+      real*8 par(lpar),w(m),siq(n),g(3,n),z(n,m),res(n),resd(n),
+     1       dkgj(n,m),dkgj2(n,m),ddkdphig(n,m),ddkdetag(n,m),
+     2       dzdpars(n,m,3),work1(n,m),work2(n,m),dfdparw(1)
+      integer i,j
+      real*8 th,m2th,sphi,cphi,seta,ceta,z1,z2,p0,p1,d1,d2,
+     1       dphi1,dphi2
+      real*8 ddot
+      external ddot
+      th = par(1)
+      m2th = -2.d0*th
+      DO i = 1,m
+         p0=par(2*i)
+         p1=par(2*i+1)
+         sphi = sin(p0)
+         cphi = cos(p0)
+         seta = sin(p1)
+         ceta = cos(p1)
+         d1 = sphi*ceta
+         d2 = sphi*seta
+C         d3 = cphi
+         dphi1 = cphi*ceta
+         dphi2 = cphi*seta
+C         dphi3 = -sphi
+C         deta1 = -sphi*seta
+C         deta2 = sphi*ceta
+         DO j = 1,n
+            z1 = d1*g(1,j)+d2*g(2,j)+cphi*g(3,j)
+            dkgj(j,i) = z1
+            z2 = z1*z1
+            dkgj2(j,i) = z2
+            z(j,i) = exp(-th*z2)
+            ddkdphig(j,i) = dphi1*g(1,j)+dphi2*g(2,j)-sphi*g(3,j)
+            ddkdetag(j,i) = d1*g(2,j)-d2*g(1,j)
+         END DO
+      END DO
+      call dcprod0(dkgj,ddkdphig,n*m,work1)
+      call dcprod0(dkgj,ddkdetag,n*m,work2)
+C initialize unneeded elements
+C      call zerofill(dzdpars,m*3*n)
+C compute componentswise -dkgj2*z
+      call dcprod(dkgj2,z,-1.d0,n*m,dzdpars(1,1,1))
+C compute componentswise m2th*work1*z
+      call dcprod(work1,z,m2th,n*m,dzdpars(1,1,2))
+C compute componentswise m2th*work2*z
+      call dcprod(work2,z,m2th,n*m,dzdpars(1,1,3))
+C   Compute residuals
+      DO j=1,n
+         z1 = siq(j)
+         z2 = 0.d0
+         DO i=1,m
+            z1 = z1 - w(i)*z(j,i)
+            z2 = z2 + w(i)*dzdpars(j,i,1)
+         END DO
+         res(j)=z1
+C         resd(j)=ddot(m,w,n,dzdpars(j,1,1),n)
+         resd(j)=z2
+      END DO
+C Now compute gradient
+      dfdparw(1)=-2.d0*ddot(n,res,1,resd,1)
+      DO i=1,m
+         dfdparw(2*i)=-2.d0*w(i)*ddot(n,res,1,dzdpars(1,i,2),1)
+         dfdparw(2*i+1)=-2.d0*w(i)*ddot(n,res,1,dzdpars(1,i,3),1)
+         dfdparw(2*m+1+i)=-2.d0*ddot(n,res,1,z(1,i),1)
+      END DO
+      call rchkusr()
+      RETURN
+      END 
+C
+C __________________________________________________________________
+C
+      subroutine mfunpli(par,w,siq,g,m,lpar,n,z,erg)
+C
+C   model with isotropic compartment 
+C   with explicit parametrization of weights in w(m+1)
+C
+C   code is restricted to m<=6
+C
+      implicit logical (a-z)
+      integer m,lpar,n
+      real*8 par(lpar),w(1),siq(n),g(3,n),z(n,m),erg
+      integer i,j
+      real*8 th,sw,sth,z1,p0,p1,d1,d2,d3
+      th = par(1)
+      DO i = 1,m
+         p0=par(2*i)
+         p1=par(2*i+1)
+         sth = sin(p0)
+         d1 = sth*cos(p1)
+         d2 = sth*sin(p1)
+         d3 = cos(p0)
+         DO j = 1,n
+            z1 = d1*g(1,j)+d2*g(2,j)+d3*g(3,j)
+            z(j,i) = exp(-th*z1*z1)
+         END DO
+      END DO
+      sw=0.d0
+      DO j=1,n
+         z1 = siq(j)-w(1)
+         DO i=1,m
+            z1=z1-w(i+1)*z(j,i)
+         END DO
+         sw=sw+z1*z1
+      END DO
+      erg=sw
+      call rchkusr()
+      RETURN
+      END 
+C
+C __________________________________________________________________
+C
+      subroutine gmfunpli(par,w,siq,g,m,lpar,n,z,res,resd,dkgj,
+     1           dkgj2,ddkdphig,ddkdetag,dzdpars,work1,work2,dfdparw)
+C
+C   analytical gradients for c(par,w)
+C   model with isotropic compartment 
+C   with explicit parametrization of weights
+C
+C   code is restricted to m<=6
+C
+      implicit logical (a-z)
+      integer m,lpar,n
+      real*8 par(lpar),w(m+1),siq(n),g(3,n),z(n,m),res(n),
+     1       resd(n),dkgj(n,m),dkgj2(n,m),ddkdphig(n,m),ddkdetag(n,m),
+     3       dzdpars(n,m,3),work1(n,m),work2(n,m),dfdparw(1)
+      integer i,j
+      real*8 th,m2th,sphi,cphi,seta,ceta,z1,z2,p0,p1,sres,d1,d2,
+     1       dphi1,dphi2
+      real*8 ddot
+      external ddot
+      th = par(1)
+      m2th = -2.d0*th
+      DO i = 1,m
+         p0=par(2*i)
+         p1=par(2*i+1)
+         sphi = sin(p0)
+         cphi = cos(p0)
+         seta = sin(p1)
+         ceta = cos(p1)
+         d1 = sphi*ceta
+         d2 = sphi*seta
+C         d3 = cphi
+         dphi1 = cphi*ceta
+         dphi2 = cphi*seta
+C         dphi3 = -sphi
+C         deta1 = -sphi*seta
+C         deta2 = sphi*ceta
+         DO j = 1,n
+            z1 = d1*g(1,j)+d2*g(2,j)+cphi*g(3,j)
+            dkgj(j,i) = z1
+            z2 = z1*z1
+            dkgj2(j,i) = z2
+            z(j,i) = exp(-th*z2)
+            ddkdphig(j,i) = dphi1*g(1,j)+dphi2*g(2,j)-sphi*g(3,j)
+            ddkdetag(j,i) = d1*g(2,j)-d2*g(1,j)
+         END DO
+      END DO
+      call dcprod0(dkgj,ddkdphig,n*m,work1)
+      call dcprod0(dkgj,ddkdetag,n*m,work2)
+C initialize unneeded elements
+C      call zerofill(dzdpars,m*3*n)
+C compute componentswise -dkgj2*z
+      call dcprod(dkgj2,z,-1.d0,n*m,dzdpars(1,1,1))
+C compute componentswise m2th*work1*z
+      call dcprod(work1,z,m2th,n*m,dzdpars(1,1,2))
+C compute componentswise m2th*work2*z
+      call dcprod(work2,z,m2th,n*m,dzdpars(1,1,3))
+C   Compute residuals
+      sres=0.d0
+      DO j=1,n
+         z1 = siq(j)-w(1)
+         z2 = 0.d0
+         DO i=1,m
+            z1 = z1 - w(i+1)*z(j,i)
+            z2 = z2 + w(i+1)*dzdpars(j,i,1)
+         END DO
+         res(j) = z1
+         resd(j) = z2
+         sres=sres+z1
+C         resd(j)=ddot(m,w(2),1,dzdpars(j,1,1),n)
+      END DO
+C Now compute gradient
+      dfdparw(1)=-2.d0*ddot(n,res,1,resd,1)
+      dfdparw(2*m+2)=-2.d0*sres
+      DO i=1,m
+         dfdparw(2*i)=-2.d0*w(i+1)*ddot(n,res,1,dzdpars(1,i,2),1)
+         dfdparw(2*i+1)=-2.d0*w(i+1)*ddot(n,res,1,dzdpars(1,i,3),1)
+         dfdparw(2*m+2+i)=-2.d0*ddot(n,res,1,z(1,i),1)
+      END DO
+      call rchkusr()
+      RETURN
+      END 
+C
+C __________________________________________________________________
+C
+      subroutine mfunpl0(par,siq,g,m,lpar,n,pen,z,w,erg)
 C
 C   model without isotropic compartment 
 C   same as mfunpl but with unconstrained least squares for weights
@@ -9,22 +254,21 @@ C
 C   code is restricted to m<=6
 C
       implicit logical (a-z)
-      integer m,lpar,ngrad
-      real*8 par(lpar),siq(ngrad),grad(3,ngrad),z(ngrad,m),erg,pen
-      integer i,j,i3,mode,jpvt(6),rank
-      real*8 th,w(ngrad),sw,sth,z1,p0,p1,d1,d2,d3,work(25)
+      integer m,lpar,n
+      real*8 par(lpar),siq(n),g(3,n),z(n,m),erg,pen
+      integer i,j,mode,jpvt(6),rank
+      real*8 th,w(n),sw,sth,z1,p0,p1,d1,d2,d3,work(25)
       th = par(1)
       th = max(th,-5.d0)
       DO i = 1,m
-         i3=2*i
-         p0=par(i3)
-         p1=par(i3+1)
+         p0=par(2*i)
+         p1=par(2*i+1)
          sth = sin(p0)
          d1 = sth*cos(p1)
          d2 = sth*sin(p1)
          d3 = cos(p0)
-         DO j = 1,ngrad
-            z1 = d1*grad(1,j)+d2*grad(2,j)+d3*grad(3,j)
+         DO j = 1,n
+            z1 = d1*g(1,j)+d2*g(2,j)+d3*g(3,j)
             z(j,i) = exp(-th*z1*z1)
          END DO
          jpvt(i)=0
@@ -32,8 +276,8 @@ C
 C  
 C    siq will be replaced, need to copy it if C-version of optim is used
 C
-      call dcopy(ngrad,siq,1,w,1)
-      call dgelsy(ngrad,m,1,z,ngrad,w,ngrad,jpvt,1d-6,rank,work,25,
+      call dcopy(n,siq,1,w,1)
+      call dgelsy(n,m,1,z,n,w,n,jpvt,1d-8,rank,work,25,
      1            mode)
 C  1d-6  is a limit for condition number 
       IF(mode.ne.0) THEN
@@ -47,7 +291,7 @@ C penalize for negative weights
          DO i=1,m
             if(w(i).lt.0.d0) sw=sw-pen*w(i)
          END DO
-         DO i=m+1,ngrad
+         DO i=m+1,n
             sw=sw+w(i)**2
          END DO
          erg=sw
@@ -58,7 +302,7 @@ C penalize for negative weights
 C
 C __________________________________________________________________
 C
-      subroutine mfunpl0h(par,siq,grad,m,lpar,ngrad,z,w,b,
+      subroutine mfunpl0h(par,siq,g,m,lpar,n,z,w,b,
      1                    work1,erg)
 C
 C   model without isotropic compartment using Larsson-Hansson code
@@ -67,103 +311,38 @@ C
 C   code is restricted to m<=6
 C
       implicit logical (a-z)
-      integer m,lpar,ngrad
-      real*8 par(lpar),siq(ngrad),grad(3,ngrad),z(ngrad,m),erg,
-     1       b(ngrad),work1(ngrad)
-      integer i,j,i3,mode,ind(1000)
-      real*8 th,w(ngrad),sth,z1,p0,p1,d1,d2,d3,work2(10)
+      integer m,lpar,n
+      real*8 par(lpar),siq(n),g(3,n),z(n,m),erg,b(n),work1(n)
+      integer i,j,mode,ind(1000)
+      real*8 th,w(n),sth,z1,p0,p1,d1,d2,d3,work2(10)
       th = par(1)
       th = max(th,-5.d0)
       DO i = 1,m
-         i3=2*i
-         p0=par(i3)
-         p1=par(i3+1)
+         p0=par(2*i)
+         p1=par(2*i+1)
          sth = sin(p0)
          d1 = sth*cos(p1)
          d2 = sth*sin(p1)
          d3 = cos(p0)
-         DO j = 1,ngrad
-            z1 = d1*grad(1,j)+d2*grad(2,j)+d3*grad(3,j)
+         DO j = 1,n
+            z1 = d1*g(1,j)+d2*g(2,j)+d3*g(3,j)
             z(j,i) = exp(-th*z1*z1)
          END DO
       END DO
 C  
 C    siq will be replaced, need to copy it if C-version of optim is used
 C
-      call dcopy(ngrad,siq,1,b,1)
-      call nnls(z,ngrad,ngrad,m,b,w,erg,work2,work1,ind,mode)
+      call dcopy(n,siq,1,b,1)
+      call nnls(z,n,n,m,b,w,erg,work2,work1,ind,mode)
       IF(mode.eq.2) erg = 1d10
+      if(th.lt.0.d0) erg=erg-1.d2*th
       call rchkusr()
       RETURN
       END 
 C
 C __________________________________________________________________
 C
-      subroutine mfunpl1(par,siq,grad,mp1,lpar,ngrad,pen,z,w,erg)
-C
-C   model with isotropic compartment 
-C
-C   code is restricted to m<=6
-C
-      implicit logical (a-z)
-      integer m,mp1,lpar,ngrad
-      real*8 par(lpar),siq(ngrad),grad(3,ngrad),z(ngrad,mp1),erg,pen
-      integer i,j,i3,mode,jpvt(6),rank
-      real*8 th,w(ngrad),sw,sth,z1,p0,p1,d1,d2,d3,work(36),eth
-      m = mp1-1
-      th = par(1)
-      th = max(th,-5.d0)
-      eth = exp(-th)
-      DO j = 1,ngrad
-         z(j,1) = eth
-      END DO
-      DO i = 1,m
-C maximal m-1 components
-         i3=2*i
-         p0=par(i3)
-         p1=par(i3+1)
-         sth = sin(p0)
-         d1 = sth*cos(p1)
-         d2 = sth*sin(p1)
-         d3 = cos(p0)
-         DO j = 1,ngrad
-            z1 = d1*grad(1,j)+d2*grad(2,j)+d3*grad(3,j)
-            z(j,i+1) = exp(-th*z1*z1)
-         END DO
-      END DO
-C  
-C    siq will be replaced, need to copy it if C-version of optim is used
-C
-      DO i=1,mp1
-         jpvt(i)=0
-      END DO
-      call dcopy(ngrad,siq,1,w,1)
-      call dgelsy(ngrad,mp1,1,z,ngrad,w,ngrad,jpvt,1d-6,rank,work,36,
-     1            mode)
-C  1d-6  is a limit for condition number 
-      IF(mode.ne.0) THEN
-         call intpr("mode",4,mode,1)
-      ELSE
-         sw=0.d0
-C penalize for extreme th values
-         if(th.gt.1.d1) sw=sw+th-1.d1
-C penalize for negative weights
-         if(th.lt.1.d-2) sw=sw-1.d2*th+1.d0
-         DO i=1,mp1
-            if(w(i).lt.0.d0) sw=sw-pen*w(i)
-         END DO
-         DO i=mp1+1,ngrad
-            sw=sw+w(i)**2
-         END DO
-         erg=sw
-      END IF
-      call rchkusr()
-      RETURN
-      END 
-C
-C __________________________________________________________________
-C
-      subroutine mfunpl0w(par,w,siq,grad,m,lpar,ngrad,z,erg)
+      subroutine mfunpl0w(par,w,siq,g,m,lpar,n,z,erg)
 C
 C   model without isotropic compartment 
 C   same as mfunpl but with unconstrained least squares for weights
@@ -171,27 +350,26 @@ C
 C   code is restricted to m<=6
 C
       implicit logical (a-z)
-      integer m,lpar,ngrad
-      real*8 par(lpar),siq(ngrad),grad(3,ngrad),z(ngrad,m),w(m)
-      integer i,j,i3
+      integer m,lpar,n
+      real*8 par(lpar),siq(n),g(3,n),z(n,m),w(m)
+      integer i,j
       real*8 th,sth,z1,p0,p1,d1,d2,d3,rss,res,erg
       th = par(1)
       th = max(th,-5.d0)
       DO i = 1,m
-         i3=2*i
-         p0=par(i3)
-         p1=par(i3+1)
+         p0=par(2*i)
+         p1=par(2*i+1)
          sth = sin(p0)
          d1 = sth*cos(p1)
          d2 = sth*sin(p1)
          d3 = cos(p0)
-         DO j = 1,ngrad
-            z1 = d1*grad(1,j)+d2*grad(2,j)+d3*grad(3,j)
+         DO j = 1,n
+            z1 = d1*g(1,j)+d2*g(2,j)+d3*g(3,j)
             z(j,i) = exp(-th*z1*z1)
          END DO
       END DO
       rss =0.d0
-      DO j=1,ngrad
+      DO j=1,n
          res=siq(j)
          DO i=1,m
             res=res-w(i)*z(j,i)
@@ -205,56 +383,8 @@ C
 C
 C __________________________________________________________________
 C
-      subroutine mfunpl1w(par,w,siq,grad,mp1,lpar,ngrad,z,erg)
-C
-C   model with isotropic compartment 
-C
-C   code is restricted to m<=6
-C
-      implicit logical (a-z)
-      integer m,mp1,lpar,ngrad
-      real*8 par(lpar),w(mp1),siq(ngrad),grad(3,ngrad),z(ngrad,mp1),
-     1       erg
-      integer i,j,i3
-      real*8 th,sth,z1,p0,p1,d1,d2,d3,eth,rss,res
-      m = mp1-1
-      th = par(1)
-      th = max(th,-5.d0)
-      eth = exp(-th)
-      DO j = 1,ngrad
-         z(j,1) = eth
-      END DO
-      DO i = 1,m
-C maximal m-1 components
-         i3=2*i
-         p0=par(i3)
-         p1=par(i3+1)
-         sth = sin(p0)
-         d1 = sth*cos(p1)
-         d2 = sth*sin(p1)
-         d3 = cos(p0)
-         DO j = 1,ngrad
-            z1 = d1*grad(1,j)+d2*grad(2,j)+d3*grad(3,j)
-            z(j,i+1) = exp(-th*z1*z1)
-         END DO
-      END DO
-      rss =0.d0
-      DO j=1,ngrad
-         res=siq(j)
-         DO i=1,mp1
-            res=res-w(i)*z(j,i)
-         END DO
-         rss=rss+res*res
-      END DO
-      erg=rss
-      call rchkusr()
-      RETURN
-      END 
-C
-C __________________________________________________________________
-C
-      subroutine mfpl0gn(par,siq,grad,m,lpar,ngrad,pen,eps,z,w,
-     1                   para,parb,dfdpar)
+      subroutine mfpl0gn(par,siq,g,m,lpar,n,pen,eps,z,w,para,parb,
+     1                   dfdpar)
 C
 C   model without isotropic compartment 
 C   same as mfunpl but with unconstrained least squares for weights
@@ -262,9 +392,9 @@ C
 C   code is restricted to m<=6
 C
       implicit logical (a-z)
-      integer m,lpar,ngrad
-      real*8 par(lpar),siq(ngrad),grad(3,ngrad),z(ngrad,m),pen,
-     1       dfdpar(lpar),para(lpar),parb(lpar),eps
+      integer m,lpar,n
+      real*8 par(lpar),siq(n),g(3,n),z(n,m),pen,dfdpar(lpar),
+     1       para(lpar),parb(lpar),eps
       real*8 erga,ergb,deltai
       integer i
       deltai=0.5d0/eps
@@ -273,8 +403,8 @@ C
          call dcopy(lpar,par,1,parb,1)
          para(i)=para(i)-eps
          parb(i)=parb(i)+eps
-         call mfunpl0(para,siq,grad,m,lpar,ngrad,pen,z,w,erga)
-         call mfunpl0(parb,siq,grad,m,lpar,ngrad,pen,z,w,ergb)
+         call mfunpl0(para,siq,g,m,lpar,n,pen,z,w,erga)
+         call mfunpl0(parb,siq,g,m,lpar,n,pen,z,w,ergb)
          if(max(ergb,erga).lt.1d10) THEN
             dfdpar(i)=(ergb-erga)*deltai
          ELSE
@@ -286,53 +416,18 @@ C
 C
 C __________________________________________________________________
 C
-      subroutine mfpl0hgn(par,siq,grad,m,lpar,ngrad,eps,z,w,b,
-     1                    work1,para,parb,dfdpar)
-C
-C   model without isotropic compartment 
-C   same as mfunpl but with unconstrained least squares for weights
-C
-C   code is restricted to m<=6
-C
-      implicit logical (a-z)
-      integer m,lpar,ngrad
-      real*8 par(lpar),siq(ngrad),grad(3,ngrad),z(ngrad,m),b(lpar),
-     1       dfdpar(lpar),para(lpar),parb(lpar),eps,work1(ngrad)
-      real*8 erga,ergb,deltai
-      integer i
-      deltai=0.5d0/eps
-      DO i=1,lpar
-         call dcopy(lpar,par,1,para,1)
-         call dcopy(lpar,par,1,parb,1)
-         para(i)=para(i)-eps
-         parb(i)=parb(i)+eps
-         call mfunpl0h(para,siq,grad,m,lpar,ngrad,z,w,b,work1,erga)
-         call mfunpl0h(parb,siq,grad,m,lpar,ngrad,z,w,b,work1,ergb)
-         if(max(ergb,erga).lt.1d10) THEN
-            dfdpar(i)=(ergb-erga)*deltai
-         ELSE
-            dfdpar(i)=0.d0
-         ENDIF
-      END DO
-      RETURN
-      END
-C
-C __________________________________________________________________
-C
-      subroutine mfunpl0g(par,s,g,m,lpar,n,d,z,v,w,dkgj,dkgj2,
-     1                    ddkdphig,ddkdetag,dddphi,
-     2                    dddeta,dvdth,dvdphi,dvdeta,dzdpars,
-     3                    dwdpars,dwdpars2,zs,work1,
-     3                    work2,scopy,pen,dfdpar)
+      subroutine mfunpl0g(par,s,g,m,lpar,n,z,v,w,dkgj,dkgj2,ddkdphig,
+     1                  ddkdetag,dvdth,dvdphi,dvdeta,dzdpars,dwdpars,
+     2                  dwdpars2,zs,work1,work2,scopy,pen,dfdpar)
       implicit logical (a-z)
       integer m,n,lpar
-      real*8 par(lpar),s(n),g(3,n),d(3,m),z(n,m),v(m,m),dkgj(n,m),
-     1       w(n),dkgj2(n,m),ddkdphig(n,m),ddkdetag(n,m),dddphi(3,m),
-     2       dddeta(3,m),dvdth(m,m),dvdphi(m,m,m),dvdeta(m,m,m),
+      real*8 par(lpar),s(n),g(3,n),z(n,m),v(m,m),dkgj(n,m),
+     1       w(n),dkgj2(n,m),ddkdphig(n,m),ddkdetag(n,m),
+     2       dvdth(m,m),dvdphi(m,m,m),dvdeta(m,m,m),
      3       dzdpars(n,m,3),dwdpars(m,lpar),dwdpars2(m,lpar),
      4       zs(n,m),dfdpar(lpar),pen,rcond,ferr(11),berr(11)
-      integer i,j,k,l,i3,ind(5),mode,jpvt(5),rank
-      real*8 th,sw,sphi,cphi,seta,ceta,z1,z2,p0,p1,
+      integer i,j,k,l,ind(5),mode,jpvt(5),rank
+      real*8 th,sphi,cphi,seta,ceta,z1,z2,p0,p1,d1,d2,dphi1,dphi2,
      1       work(250),work1(n,m),work2(n,m),scopy(n),m2th,af(25)
 C  length of work needs to be larger or equal max(m*m,ngrad)
       real*8 ddot
@@ -340,36 +435,33 @@ C  length of work needs to be larger or equal max(m*m,ngrad)
       th = par(1)
       th = max(-5.d0,th)
       m2th = -2.d0*th
-      sw = 0
 C
 C  get d, dkgj, dkgj2, dddphi, dddeta, z, ddkdphig, ddkdetag
 C
       DO i = 1,m
-         i3=2*i
-         p0=par(i3)
-         p1=par(i3+1)
+         p0=par(2*i)
+         p1=par(2*i+1)
          sphi = sin(p0)
          cphi = cos(p0)
          seta = sin(p1)
          ceta = cos(p1)
-         d(1,i) = sphi*ceta
-         d(2,i) = sphi*seta
-         d(3,i) = cphi
-         dddphi(1,i) = cphi*ceta
-         dddphi(2,i) = cphi*seta
-         dddphi(3,i) = -sphi
-         dddeta(1,i) = -sphi*seta
-         dddeta(2,i) = sphi*ceta
-         dddeta(3,i) = 0.d0
+         d1 = sphi*ceta
+         d2 = sphi*seta
+C         d3 = cphi
+         dphi1 = cphi*ceta
+         dphi2 = cphi*seta
+C         dphi3 = -sphi
+C         deta1 = -sphi*seta
+C         deta2 = sphi*ceta
          DO j = 1,n
-            z1 = ddot(3,d(1,i),1,g(1,j),1)
+            z1 = d1*g(1,j)+d2*g(2,j)+cphi*g(3,j)
             dkgj(j,i) = z1
             z2 = z1*z1
             dkgj2(j,i) = z2
             z(j,i) = exp(-th*z2)
             zs(j,i) = z(j,i)*s(j)
-            ddkdphig(j,i) = ddot(3,dddphi(1,i),1,g(1,j),1)
-            ddkdetag(j,i) = ddot(3,dddeta(1,i),1,g(1,j),1)
+            ddkdphig(j,i) = dphi1*g(1,j)+dphi2*g(2,j)-sphi*g(3,j)
+            ddkdetag(j,i) = d1*g(2,j)-d2*g(1,j)
          END DO
       END DO
       DO i=1,m
@@ -381,7 +473,7 @@ C   next w
 C
       call dcopy(n,s,1,w,1)
       call dcopy(n*m,z,1,work1,1)
-      call dgelsy(n,m,1,work1,n,w,n,jpvt,1d-6,rank,work,25,mode)
+      call dgelsy(n,m,1,work1,n,w,n,jpvt,1d-8,rank,work,25,mode)
 C  1d-6  is a limit for condition number 
       IF(mode.gt.1) THEN
          call intpr("mode",4,mode,1)
@@ -394,7 +486,7 @@ C   componentwise products
       call dcprod0(dkgj,ddkdphig,n*m,work1)
       call dcprod0(dkgj,ddkdetag,n*m,work2)
 C initialize unneeded elements
-      call zerofill(dzdpars,m*3*n)
+C      call zerofill(dzdpars,m*3*n)
 C compute componentswise -dkgj2*z
       call dcprod(dkgj2,z,-1.d0,n*m,dzdpars(1,1,1))
 C compute componentswise m2th*work1*z
@@ -449,7 +541,7 @@ C
       IF(mode.ne.0.or.rcond.lt.1d-8) THEN
 C   solving the linear system fails due to renk deficiency of v
 C compute numerical gradients instead
-        call mfpl0gn(par,s,g,m,lpar,n,pen,1d-6,z,w,work1,work2,dfdpar)
+        call mfpl0gn(par,s,g,m,lpar,n,pen,1d-8,z,w,work1,work2,dfdpar)
          RETURN
       END IF
 C
@@ -513,265 +605,6 @@ C
       END 
 C
 C __________________________________________________________________
-C
-      subroutine mfpl1gn(par,siq,grad,m,lpar,ngrad,pen,eps,z,w,
-     1                   para,parb,dfdpar)
-C
-C   model with isotropic compartment ( m = #comp +1 )
-C   same as mfunpl but with unconstrained least squares for weights
-C
-C   code is restricted to m<=6
-C
-      implicit logical (a-z)
-      integer m,lpar,ngrad
-      real*8 par(lpar),siq(ngrad),grad(3,ngrad),z(ngrad,m),pen,
-     1       dfdpar(lpar),para(lpar),parb(lpar),eps
-      real*8 erga,ergb,deltai
-      integer i
-      deltai=0.5d0/eps
-      DO i=1,lpar
-         call dcopy(lpar,par,1,para,1)
-         call dcopy(lpar,par,1,parb,1)
-         para(i)=para(i)-eps
-         parb(i)=parb(i)+eps
-         call mfunpl1(para,siq,grad,m,lpar,ngrad,pen,z,w,erga)
-         call mfunpl1(parb,siq,grad,m,lpar,ngrad,pen,z,w,ergb)
-         if(max(ergb,erga).lt.1d10) THEN
-            dfdpar(i)=(ergb-erga)*deltai
-         ELSE
-            dfdpar(i)=0.d0
-         ENDIF
-      END DO
-      RETURN
-      END
-C
-C __________________________________________________________________
-C
-      subroutine mfunpl1g(par,s,g,m,mp1,lpar,n,d,z,v,w,dkgj,dkgj2,
-     1                    ddkdphig,ddkdetag,dddphi,dddeta,dvdth,
-     2                    dvdphi,dvdeta,dzdpars,dwdpars,dwdpars2,
-     3                    zs,work1,work2,scopy,pen,dfdpar)
-      implicit logical (a-z)
-      integer m,mp1,n,lpar
-      real*8 par(lpar),s(n),g(3,n),d(3,m),z(n,mp1),v(mp1,mp1),
-     1       dkgj(n,m),w(n),dkgj2(n,m),ddkdphig(n,m),ddkdetag(n,m),
-     2       dddphi(3,m),dddeta(3,m),dvdth(mp1,mp1),dvdphi(mp1,mp1,m),
-     3       dvdeta(mp1,mp1,m),dzdpars(n,mp1,3),dwdpars(mp1,lpar),
-     4       dwdpars2(mp1,lpar),zs(n,mp1),dfdpar(lpar),pen
-      integer i,j,k,l,i3,ind(6),mode,ip1,kp1,lp1,jpvt(6),rank
-      real*8 th,sw,sphi,cphi,seta,ceta,z1,z2,p0,p1,work(250),rcond,
-     1       work1(n,mp1),work2(n,mp1),scopy(n),zji,m2th,eth,af(36),
-     2       ferr(12),berr(12)
-      real*8 ddot
-      external ddot
-      th = par(1)
-      th = max(-5.d0,th)
-      eth=exp(-th)
-      m2th = -2.d0*th
-      sw = 0
-C
-C  get d, dkgj, dkgj2, dddphi, dddeta, z, ddkdphig, ddkdetag
-C
-      DO i = 1,m
-         ip1=i+1
-         i3=2*i
-         p0=par(i3)
-         p1=par(i3+1)
-         sphi = sin(p0)
-         cphi = cos(p0)
-         seta = sin(p1)
-         ceta = cos(p1)
-         d(1,i) = sphi*ceta
-         d(2,i) = sphi*seta
-         d(3,i) = cphi
-         dddphi(1,i) = cphi*ceta
-         dddphi(2,i) = cphi*seta
-         dddphi(3,i) = -sphi
-         dddeta(1,i) = -sphi*seta
-         dddeta(2,i) = sphi*ceta
-         dddeta(3,i) = 0.d0
-         DO j = 1,n
-            z1 = ddot(3,d(1,i),1,g(1,j),1)
-            dkgj(j,i) = z1
-            z2 = z1*z1
-            dkgj2(j,i) = z2
-            zji=exp(-th*z2)
-            z(j,ip1) = zji
-            zs(j,ip1) = zji*s(j)
-            ddkdphig(j,i) = ddot(3,dddphi(1,i),1,g(1,j),1)
-            ddkdetag(j,i) = ddot(3,dddeta(1,i),1,g(1,j),1)
-         END DO
-      END DO
-C  isotrope part in z
-      DO i=1,n
-         z(i,1) = eth
-         zs(i,1) = eth*s(i)
-      END DO
-C  
-C   we now have d, dkgj,dddphi, dddeta, z, ddkdphig, ddkdetag
-C   next w
-C
-      call dcopy(n,s,1,w,1)
-      call dcopy(n*mp1,z,1,work1,1)
-      call dgelsy(n,mp1,1,work1,n,w,n,jpvt,1d-6,rank,work,36,mode)
-      IF(mode.gt.1) THEN
-         call intpr("mode",4,mode,1)
-      END IF
-C
-C   thats weights in w now V, dVdth, dVdphi, dVdeta, dzdpars
-C
-C   use work1, work2 and scopy for intermediate results
-C   componentwise products
-      call dcprod0(dkgj,ddkdphig,n*m,work1)
-      call dcprod0(dkgj,ddkdetag,n*m,work2)
-C initialize unneeded elements
-      call zerofill(dzdpars,mp1*3*n)
-C  derivatives of z(,1) are zero (isotrop part) dzdpars(.,1,.)=0
-      DO j=1,n
-         dzdpars(j,1,1)=-eth
-      END DO
-C compute componentswise -dkgj2*z(,+1)
-      call dcprod(dkgj2,z(1,2),-1.d0,n*m,dzdpars(1,2,1))
-C compute componentswise m2th*work1*z(,+1)
-      call dcprod(work1,z(1,2),m2th,n*m,dzdpars(1,2,2))
-C compute componentswise m2th*work2*z(,+1)
-      call dcprod(work2,z(1,2),m2th,n*m,dzdpars(1,2,3))
-C      DO k=1,m
-C         kp1=k+1
-C         DO j=1,n
-C            dzdpars(j,kp1,1)=-dkgj2(j,k)*z(j,kp1)
-C            dzdpars(j,kp1,kp1)=m2th*work1(j,k)*z(j,kp1)
-C            dzdpars(j,kp1,m+kp1)=m2th*work2(j,k)*z(j,kp1)
-C         END DO
-C      END DO
-C  derivatives of v with respect to theta
-      DO k=1,mp1
-         v(k,k)=ddot(n,z(1,k),1,z(1,k),1)
-         z1 = ddot(n,dzdpars(1,k,1),1,z(1,k),1)
-         dVdth(k,k) = 2.d0*z1
-         DO l=k+1,mp1
-            z2=ddot(n,z(1,k),1,z(1,l),1)
-            v(l,k)=z2
-            v(k,l)=z2
-            z2=ddot(n,dzdpars(1,l,1),1,z(1,k),1)+
-     1         ddot(n,dzdpars(1,k,1),1,z(1,l),1)
-            dVdth(k,l) = z2
-            dVdth(l,k) = z2
-         END DO
-      END DO
-C  derivatives of v with respect to theta 
-      call zerofill(dVdphi,m*mp1*mp1)
-      call zerofill(dVdeta,m*mp1*mp1)
-      DO k=1,m
-         kp1=k+1
-         DO i=1,m
-            ip1=i+1
-            z2 = ddot(n,dzdpars(1,kp1,2),1,z(1,ip1),1)
-            dVdphi(ip1,kp1,k) = dVdphi(ip1,kp1,k)+ z2
-            dVdphi(kp1,ip1,k) = dVdphi(kp1,ip1,k)+ z2
-            z2 = ddot(n,dzdpars(1,kp1,3),1,z(1,ip1),1)
-            dVdeta(ip1,kp1,k) = dVdeta(ip1,kp1,k)+ z2
-            dVdeta(kp1,ip1,k) = dVdeta(kp1,ip1,k)+ z2
-         END DO
-      END DO
-C
-C   thats V, dVdth, dVdphi, dVdeta now fill dwdpars)
-C
-      DO k=1,mp1
-         dwdpars(k,1) = ddot(n,dzdpars(1,k,1),1,s,1) -
-     1                         ddot(mp1,dVdth(1,k),1,w,1)
-         kp1=k+1
-         DO l=1,m
-            lp1=l+1
-            dwdpars(k,lp1) = - ddot(mp1,dVdphi(1,k,l),1,w,1)
-            dwdpars(k,m+lp1) = - ddot(mp1,dVdeta(1,k,l),1,w,1)
-         END DO
-         IF(k.ne.1) THEN
-         dwdpars(k,k) = dwdpars(k,k) +
-     1                    ddot(n,dzdpars(1,k,2),1,s,1)
-         dwdpars(k,m+k) = dwdpars(k,m+k) +
-     1                    ddot(n,dzdpars(1,k,3),1,s,1)
-         END IF
-      END DO
-C
-C   thats  dzdpars  now compute  dw/dpar in dwdpars
-C
-      call dcopy(mp1*lpar,dwdpars,1,dwdpars2,1)
-      call dsysvx("N","U",mp1,lpar,v,mp1,af,mp1,ind,dwdpars2,mp1,
-     1            dwdpars,mp1,rcond,ferr,berr,work,36,jpvt,mode)
-      IF(mode.ne.0.or.rcond.lt.1d-8) THEN
-C   solving the linear system fails due to renk deficiency of v
-C compute numerical gradients instead
-      call mfpl1gn(par,s,g,mp1,lpar,n,pen,1d-6,z,w,work1,work2,dfdpar)
-         RETURN
-      END IF
-C
-C   now we have dw/dpar in dzdpars next residuals in scopy
-C
-      DO j=1,n
-         z1 = s(j)
-         z2=0.d0
-         DO k=1,mp1
-            z1 = z1 - w(k)*z(j,k)
-            z2=z2+w(k)*dzdpars(j,k,1)+dwdpars(k,1)*z(j,k)
-         END DO
-         scopy(j) = z1
-         work(j)=z2
-      END DO
-C
-C   now we have residuals in scopy compute gradient of f
-C   use work for intermediate results
-C   z(j,1)=0 und dzdpars(j,1,1) = 0
-      dfdpar(1)=-2.d0*ddot(n,work,1,scopy,1)
-      if(th.gt.1.d1) dfdpar(1)=dfdpar(1)+1.d0
-      if(th.lt.1d-2) dfdpar(1)=dfdpar(1)-1.d2
-      DO k=1,mp1
-         if(w(k).lt.0.d0) dfdpar(1)=dfdpar(1)-pen*dwdpars(k,1)
-      END DO
-C    thats derivative with respect to theta
-C 
-C    dzdpars contains dw/dpars  dzdpars(j,1,.)=0 
-C
-      DO i = 1,m
-         ip1=i+1
-         DO j = 1,n
-            z1=w(ip1)*dzdpars(j,ip1,2)
-            DO k=1,mp1
-               z1=z1+dwdpars(k,ip1)*z(j,k)
-            END DO
-            work(j)=z1
-         END DO
-         dfdpar(2*i)=-2.d0*ddot(n,work,1,scopy,1)
-         DO k=1,mp1
-            if(w(k).lt.0) dfdpar(2*i)=dfdpar(2*i)-
-     1                                    pen*dwdpars(k,ip1)
-         END DO
-C    thats derivative with respect to phi
-         DO j = 1,n
-            z1=w(ip1)*dzdpars(j,ip1,3)
-            DO k=1,mp1
-               z1=z1+dwdpars(k,m+ip1)*z(j,k)
-            END DO
-            work(j)=z1
-         END DO
-         dfdpar(2*i+1)=-2.d0*ddot(n,work,1,scopy,1)
-         DO k=1,mp1
-            if(w(k).lt.0) dfdpar(1+2*i)=dfdpar(1+2*i)-
-     1                                pen*dwdpars(k,m+ip1)
-         END DO
-      END DO
-C
-C   thats derivative with respect to eta
-C
-      z1=w(1)
-      DO i=2,mp1
-         z1=min(z1,w(i))
-      END DO
-      call rchkusr()
-      RETURN
-      END 
-C
-C __________________________________________________________________
 C
       subroutine getsii30(si,vsi,ngrad,nvox,m,dgrad,nv,th,
      1     nth,indth,egrad,isample,ntry,sms,z,siind,mval,ns,mask)
